; General String Routine testing procedure

;  All routine must have the same call, take same input and produce same output to be properly compared


TimesToLoop equ 10    ;  *** NOTE   This is all that is really needed to get consistant timings if these procedures are followed

; procs to test
Procs textequ <OVH,lostcauz32,Anonymous32,Ratch,NightWare>
; include OVH to check that overhead is being properly removed from timing

; string lengths to test
;Tests TEXTEQU <0, 1, 9, 18, 27, 36, 1000>
Tests TEXTEQU < 1, 9, 18, 27,28,29, 36, 100, 400, 1000>   ; some of these bomb with zero length strings

PriClass textequ <REALTIME_PRIORITY_CLASS> ; priority to use for tests
;PriClass textequ <HIGH_PRIORITY_CLASS>
;PriClass textequ <NORMAL_PRIORITY_CLASS>   ; for debugging
    
.686p
.model  flat, stdcall
option  casemap :none   ; case sensitive
.nolist
include windows.inc
uselib  MACRO   libname
    include     libname.inc
    includelib  libname.lib
ENDM
uselib user32
uselib kernel32
uselib comctl32
;uselib gdi32
uselib shell32
;uselib comdlg32
uselib masm32
;uselib debug
include macros.asm

inv equ Invoke

soff Macro QuotedText:Vararg    ; returns offset to a string
Local LocalText
.data
LocalText db QuotedText,0
.code
EXITM <offset LocalText>
Endm
.listall

.data?
align 16
StartingCycle dd ?
CyclePart2    dd ?
Loops         dd ?
ExtraLoops    dd ?
Min           dd ?
OverHead      dd ?   ; overhead of doing an empty proc
.code
BeginCounter macro  ; This macro is inserted before a block of test code
    align 16    ; just to be sure all code starts with equal advantage
    invoke GetCurrentProcess
    invoke SetPriorityClass, eax, PriClass  ; use the highest priority for all the tests
    invoke GetCurrentThread
    inv SetThreadPriority,eax,THREAD_PRIORITY_TIME_CRITICAL
            
    mov Min,9999999999
    mov Loops,TimesToLoop
    mov ExtraLoops,10   ; will do several times first to let things settle out
    inv Sleep,0
ExtraLoopsLoop = $
    sub ExtraLoops,1
TimingLoop = $
    mov [StartingCycle],eax     ; just to make sure they are in the cache
    mov [StartingCycle+4],eax
    Repeat 6    ; Intel says 4 but I've found that more works a little better
    xor eax,eax
    cpuid       ; call the EAX=0 version of CPUID to clear pipeline (serialize) (finish any pending instructions)
    Endm
    rdtsc
    mov [StartingCycle],eax
    mov [StartingCycle+4],edx
endm
EndCounter macro        ; This macro is inserted after each block of test code
    xor eax,eax
    cpuid       ; flush
    rdtsc
    cmp ExtraLoops,0
    jne ExtraLoopsLoop  ; skip first several iterations
    sub eax,[StartingCycle] ; get result in EDX:EAX
    sbb edx,[StartingCycle+4]
    cmp Min,eax
    jb @f
    mov Min,eax
@@:
    sub Loops,1
    jnz TimingLoop
    invoke GetCurrentThread
    inv SetThreadPriority,eax,THREAD_PRIORITY_NORMAL
    invoke GetCurrentProcess
    invoke SetPriorityClass, eax, NORMAL_PRIORITY_CLASS ; back to normal
    inv Sleep,0 
    mov eax,Min     ; return the minimum elapsed time
    .if DoingOverhead==0
        sub eax,OverHead
    .endif
endm
;----------------------------  SETUP --------------------------------------

    MAXWIDTH  equ 20
    HDRWIDTH  equ 12
    COLWIDTH  equ 6

;    %FOR proc,<Procs>
;        proc proto :dword,:dword
;    ENDM

    MaxStr=0
    scount=0
    %For T1,<Tests> ; get maximum string size
        if T1 GT MaxStr
            MaxStr=T1
        endif
        scount=scount+1
    ENDM

.data

align 16
Str1 db MaxStr+64 dup (0)      ; what will be input string1 to routines.  The extra 64 bytes are to represent
align 16                            ;   a possible next string in the real world that we would not want wiped out.
Str2 db MaxStr+64  dup (0)      ; what will be the answer returned from the routines
Str3 db MaxStr+64  dup (0)      ; what will be the correct answer to check the routines
    

align 16
Str1Offset      dd  0   ; offset to starting byte within String1 based upon number of extra bytes being tested
Str2Offset      dd  0   ; offset to starting byte within String2 based upon number of extra bytes being tested
Str1ZeroOffset  dd  0   ; computed offset to zero byte terminator of first string for each test
Str2ZeroOffset  dd  0   ; computed offset to zero byte terminator of second string
Str1Size        dd  0   ; for those routines that cheat by not computing this themselves
ErrorFound      dd  0   ; flag that an error was found processing a proc
Junk            dd  0   ; scrath place for mem protect  
.code
align 16

;Printing routines from Phil
sxFld proc DDpointer:DWORD, rvstring:DWORD, DDwidth:DWORD, DDalign:DWORD    
    .if DDalign == 0                    ; align left?
        invoke szLeft,DDpointer,rvstring,DDwidth
        invoke szLen,rvstring
        mov edx,rvstring
    @@: cmp eax,DDwidth
        jae @F
        mov byte ptr [edx+eax],32   ;space
        inc eax
        jmp @B
    @@: mov byte ptr [edx+eax],0
    .elseif DDalign == 1                ; align right?
        invoke szLen,DDpointer
        mov edx,rvstring
        xor ecx,ecx
    @@: cmp eax,DDwidth
        jae @F
        mov byte ptr [edx+ecx],32
        inc eax
        inc ecx
        jmp @B
    @@: add edx,ecx
        invoke szLeft,DDpointer,edx,DDwidth
    .endif
    ret 4*4
sxFld endp

fld$ MACRO DDpointer,DDwidth,DDalign:=<0>       ; Default is align left
    LOCAL rvstring
    .data
        rvstring db MAXWIDTH+4 dup (0)
        align 16
    .code
    invoke sxFld,reparg(DDpointer),ADDR rvstring,DDwidth,DDalign
    EXITM <ADDR rvstring>
ENDM

CheckForError proc  ; check result string for correctness.  return no carry=ok, carry set=error
    mov eax,PNamePtr    ; pointer to pNamePtr table
    mov eax,[eax]       ; pointer to address of name
    mov eax,[eax]       ; value at address
    cmp eax,0048564fh   ; "OVH"  check for overhead proc
    je AllOk
    ;String2 must equal String3.  
    mov esi,Str2Offset
    mov edi,offset Str3
    mov ecx,Str1Size
    inc ecx     ; the zero byte terminators must also match
@@: lodsb
    cmp al,[edi]
    stc     ; assume error
    jne CheckForErrorRet
    inc edi
    loop @b
AllOk:
    clc
CheckForErrorRet:
    ret
CheckForError EndP

;----------------------------------------------------------------------------------------
;----------------------------------------------------------------------------------------
;----------------------------------------------------------------------------------------
.data
 DoingOverhead dd 0
.code

align 16

start:
    pcount=1
    %for proc,<Procs>   ; count number of routines to test
        pcount=pcount+1
    endm
.data
    align 4
    ptable dd pcount dup (0)    ; table of proc addresses
    lastproc dd 0               ; address of last proc
    pname dd pcount dup (0)     ; table of addresses of proc names for printing
.code
    mov edx,offset ptable   ; address to call
    mov ecx,offset pname    ; address of ascii name of routines for printing
    %for proc,<Procs>
        mov eax,offset proc
        mov [edx],eax       ; save address of proc to call
        mov lastproc,edx    ; save ending condition for loops
        add edx,4
        ifndef &proc&name
.data
            &proc&name db "&proc&",0    ; save the proc names for headers
.code
        endif
        mov [ecx],offset &proc&name     ; save the address of the proc name for headers
        add ecx,4
    endm

.data
    T1s dd scount dup (0)       ; lengths of the test strings
    lastT1 dd 0                 ; address of last T1 to use
.code
    mov edx,offset T1s
    %FOR T1,<Tests> ; get sizes
        mov eax,T1
        mov [edx],eax
        mov lastT1,edx
        add edx,4
    ENDM

    ; Now move the actual code to test to a common postion so all tests have the same advantage
    mov ecx,MaximumProcSize
    mov edx,offset TheProc
    inv VirtualProtect,edx,ecx,PAGE_EXECUTE_READWRITE,addr Junk ; stupid invoke
    invoke GetCurrentProcess
    mov edx,MaximumProcSize
    mov ecx,offset TheProc
    invoke FlushInstructionCache,eax,ecx,edx    ; ditto
 
;   print out the headers
    print fld$(chr$("Str1 Size"),HDRWIDTH)

    mov ebx,offset T1s
    %FOR T1,<Tests>
        print fld$(chr$("&T1&"),COLWIDTH,1)
    ENDM
    print chr$(13,10)

    print fld$(chr$("====================="),HDRWIDTH)
    %FOR T1,<Tests>
        print fld$(" =====================",COLWIDTH)
    ENDM
    print chr$(13,10)


.data?
    extracount dd ? ; step through extra bytes
    PAddr      dd ? ; address of proc to call
    PTblPtr    dd ? ; address of proc within table
    PNamePtr   dd ? ; address of proc name within table
    TPtr       dd ? ; address of number of bytes in string 1 to create
    T1Siz      dd ? ; size of string 1
    TstCnt     dd ? ; number of times we have pre-tested
.code   
    ;FOR extra=0 to 3
    mov extracount,0    ; initialize loop
    ExtraLoop:
        print chr$(13,10)
        mov ecx,extracount
        print sstr$(ecx)
        print chr$(" byte misalignment",13,10)
        ;%FOR proc,<Procs>
        mov PTblPtr,offset ptable   ; initialize loop
        mov PNamePtr,offset pname
        procsloop:
            mov eax,PTblPtr ; get address of routine to test
            mov eax,[eax]
            mov PAddr,eax
            mov eax,PNamePtr
            mov eax,[eax]
            mov ecx,[eax]
            cmp ecx,0048564fh   ; "OVH"  check for overhead proc
            jne @f
            mov DoingOverhead,1
    @@:
            print fld$(eax,HDRWIDTH)
           
           ; now move the proc to a standard location
           mov esi,PAddr
           mov ecx,MaximumProcSize
           mov edi,offset TheProc
           rep movsb
           cpuid    ; flush
            ;%FOR T1,<Tests>
            mov TPtr,offset T1s ; initialize loop
            T1loop: 
                mov ecx,TPtr                ; get length of string 1
                mov eax,[ecx]
                mov T1Siz,eax
                mov Str1Size,eax            ; save for routines that require size as a parameter

                ; create fresh test strings for each routine to use 
                mov eax,offset Str1
                add eax,extracount          ; start of string1 for testing
                mov Str1Offset,eax
                add eax,T1Siz
                mov Str1ZeroOffset,eax      ; where zero byte terminator will be

                mov eax,offset Str2
                add eax,extracount          ; start of string2 for testing
                mov Str2Offset,eax
                add eax,T1Siz
                mov Str2ZeroOffset,eax     ; where zero byte terminator will be

                mov edi,offset Str1      ; create test string values
                mov ecx,MaxStr+48         ; leave bytes at end for accidents
                mov al,33

            @@: stosb
                add al,1                    ; step through all possible characters, skipping zeros.
                cmp al,0
                jne ok1
                add al,1
            ok1:
                sub ecx,1
                jnz @b
NoData:
                mov eax,Str1ZeroOffset
                mov [eax],byte ptr 0        ; set length of string
                
                mov esi,eax
                sub esi,1                   ; now create answer
                mov edi,offset Str3
                mov ecx,T1Siz
                cmp ecx,0
                je NoData2
            @@:
                mov eax,[esi]
                stosb
                dec esi
                dec ecx
                jnz @b
NoData2:
                xor eax,eax
                stosb                       ; save terminator           
                ; run several times to verify operation before running timing test
                mov TstCnt,3
PreTest:
;               push Str2Offset
;               push Str1Offset
                mov esi,offset Str1
                mov edi,offset Str2     ; make a duplicate for those routines that work in place
                mov ecx,(MaxStr+48)/4         ; leave extra bytes at end for accidents
                rep movsd
                call TheProc         ;invoke proc,Str1Offset,Str2Offset
                call CheckForError
                jc NoGood                   ; error in routine
                sub TstCnt,1
                jnz PreTest
                
                ; now do actual time test
                BeginCounter
    ;                mov eax,Str1ZeroOffset
    ;                mov [eax],byte ptr 0    ; reset length of string
    ;               push Str2Offset         ;invoke proc,Str1Offset,Str2Offset
    ;               push Str1Offset
                    mov esi,offset Str1
                    mov edi,offset Str2     ; make a duplicate for those routines that work in place
	                mov ecx,(MaxStr+48)/4         ; leave extra bytes at end for accidents
	                rep movsd
                    call TheProc
                EndCounter
                
                cmp DoingOverhead,1
                jne @f
                mov OverHead,eax
                mov DoingOverhead,0
@@:
                push eax        ; save timing results
                call CheckForError  ; and one more check to be sure everything is still ok
                pop ebx     ; get back results
                jc NoGood
                print fld$(sstr$(ebx),COLWIDTH,1)
                jmp NextTPtr
NoGood:                
                print fld$("fail",COLWIDTH,1)            
NextTPtr:                
            ;ENDM
            add TPtr,4
            mov eax,TPtr
            cmp eax,lastT1
            jbe T1loop
            print chr$(13,10)
        ;ENDM
        add PNamePtr,4
        add PTblPtr,4
        mov eax,PTblPtr
        cmp eax,lastproc
        jbe procsloop        
    ;ENDM
    add extracount,1
    cmp extracount,3    ; doing 0-3 extra bytes
    jbe ExtraLoop
    mov   eax,input(13,10,"Press enter to exit...")
    exit


;----------------------------------------------------------------------------------------
;----------------------------------------------------------------------------------------
;----------------------------------------------------------------------------------------
; --------------------- Routines under test ---------------------------------------------
;----------------------------------------------------------------------------------------
;----------------------------------------------------------------------------------------
;----------------------------------------------------------------------------------------

;--------  All routines must be written with the same call to be fairly compared  ------- 


ProcMaxSize=0
;---------------All routines must be surrounded by this code-----------------------------
align 16
PStart = $
; --------------  Routine goes here --------------------------------------

ProcSize = $ - PStart
if ProcSize gt ProcMaxSize
   ProcMaxSize=ProcSize
endif

;---------------------------- Routine 1 ----------------------------------
align 16
PStart = $
; --------------  Routine goes here --------------------------------------
comment         * ReverseStr by lostcauz *
align       16
;##########################################################################
;OPTION PROLOGUE:NONE
;OPTION EPILOGUE:NONE
lostcauz32 proc
;    lea esi,Str1            ; load effective address of string
    mov esi,Str1Offset 
;    lea edi,str3            ; load effective address of buffer
    mov edi,Str2Offset
    
    ;mov ecx,[sizeof Str1-1] ; determine length of string less terminating '0'
    mov ecx,Str1Size
    mov ebx,ecx             ; load length for manipulation
    cmp ecx,4               ; ensure string is large enough for dword manipulations
    jb l1                   ; if not, work with one byte at a time
    and ebx,3               ; test dword alignment by checking two least significant bytes
    jz l2                   ; if zero it's 4 byte aligned
    cmp ebx,3               ; test if both lsb's set
    jnz @f                  ; next test
    sub ecx,1               ; both lsb's are set, work byte by byte until dword aligned
    mov al,[esi+ecx]        ; load last character
    sub ebx,1               ; decrement for next test
    mov [edi],al            ; save byte to beginning of buffer
    add edi,1               ; increment buffer position for next byte

@@: cmp ebx,2               ; test only bit 1
    jnz @f                  ; next test
    sub ecx,1               ; decrement string position
    mov al,[esi+ecx]        ; load byte
    sub ebx,1               ; prepare for next byte
    mov [edi],al            ; save byte to buffer
    add edi,1               ; increment buffer position

@@: cmp ebx,1               ; test only bit 0
    jnz l1                  ; next
    sub ecx,1               ; update string position
    mov al,[esi+ecx]        ; load byte
    mov [edi],al            ; store to buffer
    add edi,1               ; update buffer position
    jmp l2                  ; move on to dword manipulation
    
l1: sub ecx,1               ; string is less than 4 characters
    mov al,[esi+ecx]        ; load byte
    mov [edi],al            ; store byte
    add edi,1               ; update buffer
    or ecx,ecx              ; last byte?
    jz l3                   ; done
    jmp l1                  ; continue
    
l2: sub ecx,4               ; back up 4 bytes
    jnz @f                  ; if > 0 go to dword loading loop
    mov eax,[esi+ecx]       ; else load the only dword
    bswap eax               ; reverse dword 
    mov [edi],eax           ; load dword
    jmp l3                  ; done
       
@@: mov eax,[esi+ecx]       ; load dword 
    bswap eax               ; swap bytes
    mov [edi],eax           ; store dword
    add edi,4               ; update buffer for next dword
    sub ecx,4               ; back up to next dword
    jnz @b                  ; loop
    mov eax,[esi]           ; load last dword
    bswap eax               ; swap
    mov [edi],eax           ; store and done.  
l3: 
    ret 
lostcauz32 endp
;OPTION PROLOGUE:PrologueDef
;OPTION EPILOGUE:EpilogueDef


ProcSize = $ - PStart
if ProcSize gt ProcMaxSize
   ProcMaxSize=ProcSize
endif

;---------------------------- Routine 2 ----------------------------------
align 16
PStart = $
; --------------  Routine goes here --------------------------------------
;OPTION PROLOGUE:NONE
;OPTION EPILOGUE:NONE
;...........................................................................................
comment * ReverseStr by Anonymous *
align   16
db  13 dup (0)    ;0A4h, 24h, 0, 0, 0, 0 , 8Dh, 9Bh,  0, 0, 0, 0,90 
Anonymous32 proc
    push esi
    push edi
    mov esi,Str1Offset
    mov edi,Str2Offset
    ;mov        ecx, sizeof Str1-1
    mov     ecx, Str1Size
    xor     eax, eax
    cmp     ecx, 4
    mov     byte ptr [edi+ecx], al   ;[str2+ecx]
    jl          Bytes3
    push        ebx
@@:
    mov     edx, dword ptr [esi+eax] ;[str1+eax]  
    add     eax, 4
    add     ecx, -8
    
    add ecx,eax ; temporarily compute eax+ecx
    mov     ebx, dword ptr [esi+ecx] ;[str1+eax+ecx]
    bswap   edx
    mov     dword ptr [edi+ecx], edx ;[str2+eax+ecx]
    sub ecx,eax ; restore ecx
    
    bswap   ebx
    mov     dword ptr [edi+eax-4], ebx ;[str2+eax-4]
    jg          @b   ; from the add ecx,-8 instruction
    pop     ebx
    pop edi
    pop esi
    ret
align   16
Bytes3:
                movzx   edx, byte  ptr [esi]     ;[str1]
                cmp     ecx, 1
                mov     byte ptr [edi+ecx-1], dl ;[str2+ecx-1]
                jle         End3
                mov     dl, byte  ptr [esi+1]    ;[str1+1]
                cmp     ecx, 2
                mov     byte ptr [edi+ecx-2], dl ;[str2+ecx-2]
                jle         End3
                mov     dl, byte  ptr [esi+2]    ;[str1+2]
                mov     byte ptr [edi+ecx-3], dl ;[str2+ecx-3]
End3:
                pop edi
                pop esi
                ret
Anonymous32 endp        
;OPTION PROLOGUE:PrologueDef
;OPTION EPILOGUE:EpilogueDef


ProcSize = $ - PStart
if ProcSize gt ProcMaxSize
   ProcMaxSize=ProcSize
endif

;---------------------------- Routine 3 ----------------------------------
align 16
PStart = $
; --------------  Routine goes here --------------------------------------
;*******************************************************************************
; REVBSTR:   by Ratch   Reverses the bytes of an ASCII string.                            *
;                                                                              *
; Called by: PUSH <address of string>                                          *
;            PUSH <string length>                                              *
;            CALL Ratch                                                      *
;                                                                              *
; Returns:   Nothing                                                           *
;                                                                              *
; Notes:     This subroutine conforms with WIN32 conventions regarding         *
;            register preservation and stack balancing.                        *
;                                                                              *
; Coder:     Ratch                                                             *
;*******************************************************************************
;REVBS$ STRUC
;  RETURN    DWORD ?
;RV1$ = $
;  STRLEN    DWORD ?
;  SADR      DWORD ?
;RV2$ = $
;REVBS$ ENDS
;
;RVBS$ EQU ESP.REVBS$            ;save some typing

Ratch:                        ;it all begins here
PUSH ESI                       ;save register
PUSH EDI                       ;save register
;MOV EAX,[RVBS$.STRLEN+2*DWORD] ;string length
mov eax,Str1Size
;MOV ESI,[RVBS$.SADR+2*DWORD]   ;string start address
mov esi,Str2Offset
SHRD EDX,EAX,3                 ;mod 8
PUSH EBX                       ;save register
LEA EDI,[ESI+EAX-DWORD]        ;string end address - DWORD
SHR EDX,32-3                   ;EDX = remainder
SHR EAX,3                      ;divide by 8

.IF EDX > 3
   INC EAX                      ;increase loop count
   XOR EDX,EDX                  ;remainder is set to zero
.ENDIF 

 .IF EAX
   .REPEAT                        ;8 bytes moved each time through loop
     MOV ECX,[ESI]                ;pick up 4 bytes
     MOV EBX,[EDI]                ;pick up 4 bytes
     BSWAP ECX
     BSWAP EBX
     MOV [EDI],ECX                ;put down 4 bytes
     MOV [ESI],EBX                ;put down 4 bytes
     SUB EDI,DWORD                ;move string right end pointer
     ADD ESI,DWORD                ;move string left end pointer
     DEC EAX                      ;loop counter
   .UNTIL ZERO?
.ENDIF

 .IF EDX > 1                    ;if remainder is 2 or 3
   MOV CL,BYTE PTR [ESI]        ;pick up byte
   MOV BL,BYTE PTR [EDI+DWORD-BYTE] ;pick up byte
   MOV BYTE PTR [EDI+DWORD-BYTE],CL ;put down byte
   MOV BYTE PTR [ESI],BL        ;put down byte
.ENDIF

POP ESI                        ;restore register
POP EDI                        ;restore register
POP EBX                        ;restore register
RET ;RV2$-RV1$                  ;return home


ProcSize = $ - PStart
if ProcSize gt ProcMaxSize
   ProcMaxSize=ProcSize
endif

;---------------------------- Routine 4 ----------------------------------
align 16
PStart = $
; --------------  Routine goes here --------------------------------------
NightWare PROC ;StrAddr:DWORD,StrSize:DWORD   ; by NightWare
ALIGN 16
;db 0cch
push ebx
push esi
push edi

;mov esi,StrAddr
mov esi,Str2Offset
;mov eax,StrSize
mov eax,Str1Size
lea edi,[esi+eax]; if you use SIZEOF => lea edi,[esi+eax-1]
mov ebx,eax
and eax,00000000000000000000000000000111b
shr eax,1; to eliminate impair values
and ebx,11111111111111111111111111111000b

;test ebx,ebx; no dword ? (useless coz the previous and fix the zero flag)
jz Label2

Label1:sub edi,4; big string algo
mov edx,DWORD PTR [edi]
mov ecx,DWORD PTR [esi]
bswap ecx
bswap edx
mov DWORD PTR [esi],edx
mov DWORD PTR [edi],ecx
add esi,4
sub ebx,8
jnz Label1

Label2:test eax,eax; no byte ?
jz Label4

Label3:sub edi,1; small string algo
mov dh,BYTE PTR [edi]
mov dl,BYTE PTR [esi]
mov BYTE PTR [esi],dh
mov BYTE PTR [edi],dl
add esi,1
sub eax,1
jnz Label3
Label4:
pop edi
pop esi
pop ebx
ret
NightWare ENDP

ProcSize = $ - PStart
if ProcSize gt ProcMaxSize
   ProcMaxSize=ProcSize
endif


;---------------------------- Routine 5 ----------------------------------
align 16
PStart = $
; --------------  Routine goes here --------------------------------------

ProcSize = $ - PStart
if ProcSize gt ProcMaxSize
   ProcMaxSize=ProcSize
endif

;---------------------------- Routine 6 ----------------------------------
align 16
PStart = $
; --------------  Routine goes here --------------------------------------
mov eax,ProcMaxSize

ProcSize = $ - PStart
if ProcSize gt ProcMaxSize
   ProcMaxSize=ProcSize
endif


;-------------------------------------------------------------------------------


;----------------------------------------------------------------------------------------
;----------------------------------------------------------------------------------------
;----------------------------------------------------------------------------------------
; --------------------- End of Routines under test --------------------------------------
;----------------------------------------------------------------------------------------
;----------------------------------------------------------------------------------------
;----------------------------------------------------------------------------------------

align 16
PStart = $
OVH proc ;xStr1:dword, xstr2:dword
   ret ;8
OVH EndP 
ProcSize=$-PStart
if ProcSize gt ProcMaxSize
   ProcMaxSize=ProcSize
endif

OPTION PROLOGUE:PrologueDef
OPTION EPILOGUE:EpilogueDef
.data
MaximumProcSize dd ProcMaxSize
.code


;--------------------------  procs will be moved to here for testing  ------------------------------------
align 16
TheProc:
db ProcMaxSize+16 dup (0)

;-------------------------------------------------------------------------------

end start
